/**************************************************************************************************
  MMC_Driver
  Common functions for reading and writing to a mmc/sd card.
**************************************************************************************************/

// Include Files
#include "Common.h"
#include "__Time.h"
#include "MMC_Driver.h"
#include "UART1.h"
#include "__Lib_MmcFat16.h"

// MMC Chip Select connection
sbit Mmc_Chip_Select at LATA9_bit;
sbit Mmc_Chip_Select_Direction at TRISA9_bit;

//Variables
char FileName[32];    // File name
char Buffer[512];     // Buffer for reading file data
char LineBuf[128];    // Buffer for writing a line of data to a file
char OutBuf[128];
int BytePtr;

char OpenDate[16];
char OpenTime[16];
char CloseTime[16];

char CardOK = FALSE;

//unsigned int  Cntr1, Cntr2, Cntr3, Cntr4, Cntr5, Cntr6, Cntr7, Cntr8;


int FileSM = 0;  // State machine state




extern TimeStruct ts1;
extern UART_Busy;                    // UART busy flag






/**************************************************************************************************
  Description: Initialize the SPI and FAT 16.
  Parameters: none
  Returns: void
**************************************************************************************************/
void InitMMC() {
     unsigned int CardStatus;

     // Set up SPI for the file read
     SPI1_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, 64, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);
     Delay_ms(10);

     // Initialize the FAT library
     CardStatus = Mmc_Fat_Init();

     if (CardStatus > 0) {
         CardOK = FALSE;
         DEBUGOUT("SD card missing\r\n");
     } else {
         // Re-initialize SPI at higher speed
         SPI1_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, 8, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);
         CardOK = TRUE;
         DEBUGOUT("SPI set to high speed\r\n");
     }
     DEBUGOUT("SPI initialized\r\n");
}

/**************************************************************************************************
  Description: This function creates a file for writing with the file name passed to it.
  Parameters: pointer to a file name
  Returns: 1 - mmc card present
           0 - mmc card unavailable
**************************************************************************************************/
char CreateFile(char *filename) {
     int /*Year,*/ Month, Day, Hour, Minute, Second;
     char TDString[32];
     //unsigned int Stat;

     if (CardOK == TRUE) {
         //Year = ts1.yy;    // MikroE sprintf has trouble with char type
         Month = ts1.mo;
         Day = ts1.md;
         Hour = ts1.hh;
         Minute = ts1.mn;
         Second = ts1.ss;

         sprintf(TDString,"%04d/%02d/%02d %02d:%02d:%02d",ts1.yy,Month,Day,Hour,Minute,Second);

         if (Mmc_Fat_Exists(filename)) {     // Find existing file or create a new one
             //Stat = Mmc_Fat_Assign(filename, 0);
             Mmc_Fat_Assign(filename, 0);
             Mmc_Fat_Append();
             return 1;

         } else {
             //Stat = Mmc_Fat_Assign(filename, 0xA0);
             Mmc_Fat_Assign(filename, 0xA0);
             Mmc_Fat_Set_File_Date(ts1.yy,ts1.mo,ts1.md,ts1.hh,ts1.mn,ts1.ss); // Set file date & time info
             Mmc_Fat_Rewrite();
             return 2;
         }

     } else {
         return 0;
     }
}

/**************************************************************************************************
  Description: This function writes a line to the file from a buffer previously prepared.
  Parameters: poiner to a line buffer
  Returns: 1 - line written successfully
           0 - mmc card not present
**************************************************************************************************/
char WriteFile(char *buf) {
     if (CardOK == TRUE) {
         Mmc_Fat_Write(buf,strlen(buf));
         return 1;
     } else {
         return 0;
     }
}

/**************************************************************************************************
  Description: This function closes the file.
  Parameters: none
  Returns: 1 - file closed successfully
           0 - no assigned file
**************************************************************************************************/
char CloseFile(void) {
     if (Mmc_Fat_Close() == 0) {
         return 1;
     } else {
         return 0;
     }
}

/**************************************************************************************************
  Description: This function opens a file for reading.
  Parameters: pointer to file name
  Returns: the handle for the opened file or <0 if error occurred
**************************************************************************************************/
short OpenFile(char *filename) {
     if (CardOK == TRUE) {
         Mmc_Fat_Open(filename, FILE_READ, 0x01);
         return 1;
     } else {
         return 0;
     }
}

/**************************************************************************************************
  Description: This function reads a line of data from the opened FILE. Internal pointer is
               advanced to the next line.
  Parameters: pointer to the buffer where data is to be stored
  Returns: -1 - if an error occurred
            0 - if end of file was not reached
            1 - if end of file was reached
**************************************************************************************************/
int ReadFile(char *buf) {
     int status;
     int Count;
     char Ch;

     status = Mmc_Fat_EOF();

     if (status == 0) {
         memset(buf,0,128);  // Clear the buffer
         Count = 0;
         Ch = 0;

         while ((Ch != '\n') && (Count < 127)) {
             Mmc_Fat_Read(&Ch);
             buf[Count++] = Ch;
         }
     }
     return (status);
}



























